@@ -9,12 +9,18 @@ import threading
 from random import randrange
 from time import sleep, time
 
-import cv2
+# Just in case user won't need it
+try:
+    import cv2
+except:
+    pass
+
 from PyQt5.QtCore import pyqtProperty, pyqtSlot, pyqtSignal
 from PyQt5.QtQuick import QQuickItem
 from pyffmpeg import FFmpeg, FFprobe
 
 from .pyqt_inter_audio import QAudio
+from .audio import Audio
 from .misc import Paths
 
 
@@ -47,6 +53,7 @@ class QVideo(QQuickItem):
             'mp4', "asf", "avi", "flv",
             "gif", "mov", "3gp", "3gpp",
             "mkv", "webm"]
+        self._user_stills = False
         self._stills_content = []
         self._curr_stills_index = 0
         self._stills_len = 10000000
@@ -56,6 +63,12 @@ class QVideo(QQuickItem):
         self._seeked = False
         self._seek_frame = 0
         self._seek_calls = 0
+        # Audio
+        self._audio_inst = Audio(saveFolder=self.temp_folder)
+        self._has_audio = True
+        self._play_audio = True
+        self._sync_audio = True
+        self.auto_sync_time: int = 3
         # controls
         self._stopped = False
         self._paused = False
@@ -103,6 +116,21 @@ class QVideo(QQuickItem):
             sleep(0.1)
         else:
             self._stills_content = os.listdir(self.folder)
+            self._stills_len = len(self._stills_content)
+
+    def auto_sync_audio(self):
+        a_thread = threading.Thread(target=self._auto_sync_audio)
+        a_thread.daemon = True
+        a_thread.start()
+
+    def _auto_sync_audio(self):
+        while not self._stopped:
+            if self._paused:
+                sleep(1)
+            else:
+                seconds = int(self._total_elapsed_time / 1000)
+                self._audio_inst.seek(seconds+0.7)
+            sleep(self.auto_sync_time)
 
     def convert_to_stills(self, fileName):
         if self.sync:
@@ -237,6 +265,7 @@ class QVideo(QQuickItem):
         ext = os.path.splitext(fileName)[1][1:]
         if ext not in self._supported_vid_files:
             # stills
+            self._user_stills = True
             try:
                 os.listdir(fileName)
                 self.folder = fileName
@@ -293,15 +322,22 @@ class QVideo(QQuickItem):
 
     def _pause(self):
         self._paused = True
+        if self._sync_audio:
+            self._audio_inst._not_paused = False
 
     def _play(self, fileName):
         # play video
         self._curr_file = fileName
         if not self._same_session:
+            self._user_stills = False
             filename = self.fix_splashes(fileName)
+
             if self.is_stills(filename):
-                # stills
-                pass
+                self._stills_converted = True
+                self._append_stills_content()  # call without a thread
+                self.stills_updater()
+                return
+
             else:
                 # not stills
                 # set fps based on file
@@ -316,6 +352,9 @@ class QVideo(QQuickItem):
                 if abs(fps - self.fps) > 1:
                     self.fps = fps
 
+                if self._has_audio:
+                    self._prepare_audio_file()
+
                 self.convert_to_stills(filename)
                 self.append_stills_content()
 
@@ -324,8 +363,32 @@ class QVideo(QQuickItem):
         self.updater()
         # self.monitor() # allow this only in debug mode
 
+    def prepare_audio_file(self):
+        a_thread = threading.Thread(target=self._prepare_audio_file)
+        a_thread.daemon = True
+        a_thread.start()
+
+    def _prepare_audio_file(self):
+        fileName = self.fix_splashes(self._curr_file)
+        self._audio_inst.prepare(fileName)
+        print(self._audio_inst.file)
+
+    def play_audio_file(self, delay: float):
+        a_thread = threading.Thread(
+            target=self._play_audio_file,
+            args=[delay])
+        a_thread.daemon = True
+        a_thread.start()
+
+    def _play_audio_file(self, delay: float):
+        print('delay: ', delay)
+        self._audio_inst.delay_play(delay)
+        self.auto_sync_audio()
+
     def _resume(self):
         self._paused = False
+        if self._sync_audio:
+            self._audio_inst._not_paused = True
 
     def _seek(self, seconds):
         status = 'continue'
@@ -397,6 +460,9 @@ class QVideo(QQuickItem):
         self._seek_frame = frame_no
         self._start_time = time()
         self.setTime()
+        if self._sync_audio:
+            print('heres, ', seconds)
+            self._audio_inst.seek(seconds)
 
     def setFrameNo(self):
         # start the setTime thread
@@ -479,8 +545,65 @@ class QVideo(QQuickItem):
         os.makedirs(self.folder)
         self.cv2_updater()
 
+    def stills_updater(self):
+        # Avoid multiple playing instances
+        self._stopped = True
+        self._frame_no = 0
+        sleep(0.3)
+
+        u_thread = threading.Thread(target = self._stills_updater)
+        u_thread.daemon = True
+        u_thread.start()
+
+    def _stills_updater(self):
+
+        self._stopped = False
+        self._paused = False
+        self._seek_frame = 0
+
+        # initialize remaining delay
+        rem_delay = 0.0
+
+        # Make sure convertion is done
+        if len(self._stills_content) < 1:
+            return
+
+        if rem_delay < 0:
+            rem_delay = self._delay
+
+        # about to play
+        self.aboutToPlay.emit(rem_delay)
+
+        # Delay
+        if self._delay:
+            # sleep remaining delay
+            sleep(rem_delay)
+
+        # print(self._audio_inst.playing, 'yep')
+        self._start_time = time()  # set the universal start time
+        self.setTime()
+        self.setFrameNo()
+
+        while not self._stopped and self._frame_no != self._stills_len:
+
+            #t1 = time()
+            filename = self._stills_content[self._frame_no]  # use still type
+            if not self._paused:
+                self._current_frame = f"file:///{self.folder}/{filename}"
+                self.updateFrame('')
+                sleep(1/self.fps) # sleep equivalent of FPS
+            else:
+                sleep(1/10)
+
+        # stop showing the last frame
+        self._stopped = True  # stop all other processs; will cause no trouble
+        self._current_frame = ''
+        self.updateFrame('')
+
     def _stop(self):
         self._stopped = True
+        if self._sync_audio:
+            self._audio_inst._not_stopped = False
         self._ffmpeg_inst.quit()
 
     def updateFrame(self, frame):
@@ -494,6 +617,7 @@ class QVideo(QQuickItem):
         # we will need to reset it in order to restart play
         self._stopped = False
         self._paused = False
+        self._seek_frame = 0
 
         # initialize remaining delay
         rem_delay = 0.0
@@ -509,11 +633,21 @@ class QVideo(QQuickItem):
         # about to play
         self.aboutToPlay.emit(rem_delay)
 
+        # Play audio
+        if self._play_audio and self._has_audio:
+            self.play_audio_file(rem_delay)
+
         # Delay
         if self._delay:
             # sleep remaining delay
             sleep(rem_delay)
 
+        if self._play_audio and self._has_audio:
+            while not self._audio_inst.playing:
+                sleep(0.1)
+            sleep(0.5)
+
+        # print(self._audio_inst.playing, 'yep')
         self._start_time = time()  # set the universal start time
         self.setTime()
         self.setFrameNo()
